/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#ifndef STAF_ConnectionManager
#define STAF_ConnectionManager

#include "STAF.h"
#include "STAFString.h"
#include "STAFConnectionProvider.h"

class STAFConnectionManager
{
public:

    STAFConnectionManager();

    typedef std::deque<STAFString> ConnectionProviderOptionList;
    typedef std::map<STAFString, STAFConnectionProviderPtr>
        ConnectionProviderMap;
    typedef std::deque<STAFConnectionProviderPtr> ConnectionProviderList;

    STAFRC_t addConnectionProvider(
                 const STAFString &name,
                 const STAFString &library,
                 const ConnectionProviderOptionList &optionList,
                 STAFString &errorBuffer);

    STAFRC_t setDefaultConnectionProvider(const STAFString &name);
    STAFString getDefaultConnectionProvider();

    ConnectionProviderList getConnectionProviderListCopy();

    STAFRC_t makeConnection(const STAFString &where,
                            STAFConnectionPtr &connection,
                            STAFString &errorBuffer);

    STAFRC_t makeConnection(const STAFString &where,
                            STAFConnectionProviderPtr &provider,
                            STAFConnectionPtr &connection,
                            STAFString &errorBuffer);

    ~STAFConnectionManager();

private:

    ConnectionProviderMap fConnProvMap;
    ConnectionProviderList fConnProvList;
    STAFString fDefaultConnProv;
};

#endif



